/*
 * Управление RGB лентой из браузера.
 * подключить 4-х канальный модуль силовых ключей на 9-ом адресе
 * 1 канал - синий, 2 канал - красный, 3 канал - зелёный
 * после загрузки открыть в браузере http://rgb.local
 */

// Подключаем библиотеки
#include <WiFi.h>
#include <ESPAsyncWebServer.h>
#include <ESPmDNS.h>
#include <iarduino_I2C_Relay.h>
#include "index.h"

// Создаём объект силовых ключей Flash-I2C с установленным адресом
iarduino_I2C_Relay fets(0x09);

// Создаём глобальные переменные яркости цветов светодиодной ленты
uint8_t currR = 0;
uint8_t currG = 0;
uint8_t currB = 0;

// Настройки WiFi - необходимо ввести данные Вашего WiFi
const char* ssid = "intheair";
const char* password = "fuckth3naz1es";

// Локальный адрес сервера
const char* host = "rgb";

// Создаём объекты сервера и WebSocket
AsyncWebServer server(80);
AsyncWebSocket ws("/");

// Callback-функция при получении события WebSocket
void onWsEvent(
        AsyncWebSocket * server,
        AsyncWebSocketClient * client,
        AwsEventType type,
        void * arg,
        uint8_t *data,
        size_t len
        )
{

    // Если событие - подключение клиента
    if(type == WS_EVT_CONNECT){

        Serial.println("Подключился новый клиент");

        // Формируем строку для отправки
        String resp = (String)"{\"R\":" + currR + ","
                + "\"G\":" + currG + ","
                + "\"B\":" + currB + "}";

        delay(500);

        // Отправляем строку
        client->text(resp);


    }
    // Если событие - отключение клиента
    else if(type == WS_EVT_DISCONNECT){
        Serial.println("Клиент отключлся");
        Serial.println("-----------------------");

    }
    // Если событие - данные
    else if(type == WS_EVT_DATA){

        // Преобразуем байты в объект строки
        String received = String((char*) data);

        // Если строка начинается с интересующих нас символов
        if (received.startsWith("R:")) {
            // записываем инндекс следующей запятой
            int next_index = received.indexOf(',');
            // записываем часть строки, отвечающий за красный цвет в строку R
            String R = received.substring(received.indexOf('R')+2, next_index);
            // Преобразуем и записываем данные
            currR = (uint8_t)R.toInt();

            // Записываем индекс следующей запятой
            next_index = received.indexOf(',', received.indexOf('G'));

            // записываем часть строки, отвечающий за зелёный цвет в строку G
            String G = received.substring(received.indexOf('G')+2, next_index);
            // Преобразуем и записываем данные
            currG = (uint8_t)G.toInt();

            // Записываем индекс точки с запятой
            next_index = received.indexOf(';');

            // записываем часть строки, отвечающий за синий цвет в строку B
            String B = received.substring(received.indexOf('B')+2, next_index);
            // Преобразуем и записываем данные
            currB = (uint8_t)B.toInt();
        }
    }
}

void setup()
{
    // Инициируем работу с силовыми ключами
    fets.begin();
    // Выключаем все каналы
    fets.digitalWrite(ALL_CHANNEL, LOW);

    // Инициируем UART
    Serial.begin(115200);

    // Инициируем WiFi
    WiFi.begin(ssid, password);

    while (WiFi.status() != WL_CONNECTED) {
        delay(1000);
        Serial.println("Connecting to WiFi..");
    }

    // Выводим текущий IP адрес
    Serial.println(WiFi.localIP());

    // Инициируем Multicast DNS
    MDNS.begin(host);

    // Указываем callback функцию при получении события WebSocket
    ws.onEvent(onWsEvent);
    server.addHandler(&ws);

    // Указываем серверу действие при подключении к корневому каталогу
    server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
            request->send(200, "text/html", String(main_page));
            });

    // Инициируем сервер
    server.begin();
}

void loop()
{
    // Приводим значения глобальных переменных
    uint16_t red = map(currR, 0, 100, 0, 4095);
    uint16_t green = map(currG, 0, 100, 0, 4095);
    uint16_t blue = map(currB, 0, 100, 0, 4095);
    // Выводим ШИМ на каналы модуля
    fets.analogWrite(1, blue);
    fets.analogWrite(2, red);
    fets.analogWrite(3, green);

    // Даём процессору переключится на другие задачи
    delay(2);
}
